@@ -3,7 +3,7 @@ from typing import Callable, Optional, Any
 from aiogram.dispatcher.filters.state import State
 from aiogram.types import CallbackQuery
 
-from aiogram_dialog.manager.intent import ChatEvent
+from aiogram_dialog.context.events import ChatEvent, StartMode, Data
 from aiogram_dialog.manager.manager import DialogManager
 from aiogram_dialog.widgets.text import Text, Const
 from aiogram_dialog.widgets.widget_event import WidgetEventProcessor
@@ -12,7 +12,8 @@ from ..when import WhenCondition
 
 
 class EventProcessorButton(Button, WidgetEventProcessor):
-    async def process_event(self, event: ChatEvent, source: Any, manager: DialogManager, *args, **kwargs):
+    async def process_event(self, event: ChatEvent, source: Any, manager: DialogManager,
+                            *args, **kwargs):
         await self._on_click(event, self, manager)
 
     async def _on_click(self, c: CallbackQuery, button: Button, manager: DialogManager):
@@ -68,31 +69,35 @@ class Back(EventProcessorButton):
 
 class Cancel(EventProcessorButton):
     def __init__(self, text: Text = Const("Cancel"), id: str = "__cancel__",
+                 result: Any = None,
                  on_click: Optional[Callable] = None,
                  when: WhenCondition = None):
         super().__init__(text, id, self._on_click, when)
         self.text = text
+        self.result = result
         self.user_on_click = on_click
 
     async def _on_click(self, c: CallbackQuery, button: Button, manager: DialogManager):
         if self.user_on_click:
             await self.user_on_click(c, self, manager)
-        await manager.done()
+        await manager.done(self.result)
 
 
 class Start(EventProcessorButton):
     def __init__(self, text: Text, id: str,
                  state: State,
+                 data: Data = None,
                  on_click: Optional[OnClick] = None,
-                 reset_stack: bool = False,
+                 mode: StartMode = StartMode.NORMAL,
                  when: WhenCondition = None):
         super().__init__(text, id, self._on_click, when)
         self.text = text
+        self.start_data = data
         self.user_on_click = on_click
         self.state = state
-        self.reset_stack = reset_stack
+        self.mode = mode
 
     async def _on_click(self, c: CallbackQuery, button: Button, manager: DialogManager):
         if self.user_on_click:
             await self.user_on_click(c, self, manager)
-        await manager.start(self.state, reset_stack=self.reset_stack)
+        await manager.start(self.state, self.start_data, self.mode)
